# -*- coding: utf-8 -*-
import numpy as np
import cv2
from scipy.spatial import distance as dist
from matplotlib import pyplot as plt
import math, os
import pandas as pd


def seed_scanner(photo, border, min_obj_area, dpi = 600):
    """seed phenotyping using scanned photos of seeds
    Input photo should be a black and white scanned picture ...
    with a homogeneous background (lighter than seeds, see 'seeds.jpg')
    
    Arguments:
    - photo: path/to/input_photo.jpg
    - border: number of pixels to crop on input photo margin
    - min_obj_area:
	-- Minimum number of pixels allowed to be considered as a seed. 
	-- Under this threshold, the object is out of the analysis.
    - dpi: dots per inch of output photo.
    """

    # Load and crop input photo:
    img = cv2.imread(photo)
    # NOTE: its img[y: y + h, x: x + w] and *not* img[x: x + w, y: y + h]
         crop_img = img[border:(img.shape[0]) - border, border:(img.shape[1]) - border] 
    
    # Segmentation:
    gray = cv2.cvtColor(crop_img, cv2.COLOR_BGR2GRAY)
    ret, thresh = cv2.threshold(gray, 0, 255, cv2.THRESH_BINARY_INV+cv2.THRESH_OTSU)

    # Get contours = objects
    # cv2.CHAIN_APPROX_NONE: all the boundary points are stored
    image, contours, hier = cv2.findContours(
    	thresh, cv2.RETR_TREE, cv2.CHAIN_APPROX_NONE
    )
    contours = [c for c in contours if cv2.contourArea(c) > min_obj_area ] 

    # Create a 2D mask image only with selected objects (border and fill points): 
    # R= matrix where there is 1 on each points forming and filling the polygon, 
    # "thickness=-1" forces to fill the contour, thickness = 1 to get only the borders
    mask = np.zeros(gray.shape, dtype = "uint8")
    for c in contours:
        cv2.drawContours(mask, [c], -1, (1), thickness = -1)
    
    # To plot output picture and to export phenotype data:
    fig = plt.figure()
    ax = fig.add_axes([0,0,1,1])
    index = 0
    # data.frame
    df = pd.DataFrame(columns = ['Obj id', 'Width', 'Length', 'Area', 'Perimeter']) 
    for c in contours:
        (x0, y0), (minor_axis_length, major_axis_length), angle = cv2.fitEllipse(c)
        angle =  (90 - angle)*(math.pi/180)
        x1 = x0 + math.cos(angle) * 0.5 * major_axis_length
        y1 = y0 - math.sin(angle) * 0.5 * major_axis_length
    
        x2 = x0 - math.sin(angle) * 0.5 * minor_axis_length
        y2 = y0 - math.cos(angle) * 0.5 * minor_axis_length
    
        x3 = x0 + math.cos(math.pi + angle) * 0.5 * major_axis_length
        y3 = y0 - math.sin(math.pi + angle) * 0.5 * major_axis_length
    
        x12 = x1 + (x2 - x0)
        y12 = y1 + (y2 - y0)
    
        y23 = y3 + (y2 - y0)
        x23 = x3 - (x0 - x2)
    
        y34 = y3 - (y2 - y0)
        x34 = x3 + (x0 - x2)
    
        x41 = x1 - (x2 - x0)
        y41 = y1 - (y2 - y0)
    
        # draw  lines
        ax.plot(
	  (x23, x12), (y23, y12), 'w', linewidth = 1, zorder = 1, linestyle = ':'
        )
        ax.plot(
          (x34, x41), (y34, y41), 'w', linewidth = 1, zorder = 1, linestyle = ':'
        )
        ax.plot(
          (x41, x12), (y41, y12), 'w', linewidth = 1, zorder = 1, linestyle = ':'
        )
        ax.plot(
          (x34, x23), (y34, y23), 'w', linewidth = 1, zorder = 1, linestyle = ':'
        )
    
        # draw perimiter detected objects
        ax.plot(
          [i[0] for i in c[:, 0]], [i[1] for i in c[:, 0]], 
          'magenta', linewidth = 1.5, zorder = 2
        )
    
        # draw middle arrows
        ax.arrow(
			x0, y0, x1 - x0, y1 - y0, 
			length_includes_head = True, 
			linewidth = 1, head_width = 15, head_length = 18, 
			fc = 'red', ec = 'red', zorder = 3
		)
        ax.arrow(
			x0, y0, x2 - x0, y2 - y0, 
			length_includes_head = True, 
			linewidth = 1, head_width = 15, head_length = 18, 
			fc = 'red', ec = 'red', zorder = 3
		)
      
        # save data into 'index' row of the 'df' data.frame  
        width = minor_axis_length
        length = major_axis_length   
        area = cv2.contourArea(c) 
        perimeter = cv2.arcLength(c, True)
        df.loc[index] = [index + 1, width, length, area, perimeter]

        # identify object:
        ax.plot(int(x0), int(y0), '.k', markersize = 35, zorder = 4)
        ax.text(
	   int(x0), int(y0), str(index + 1), 
           ha = 'center', va = 'center', color = 'w', fontsize = 11, zorder = 5
        )   
        index += 1
 
    ax.imshow(
	cv2.bitwise_and(gray, gray, mask = mask), cmap = plt.cm.get_cmap('terrain')
    )
    ax.set_axis_off()
    plt.savefig(
       photo.split(".")[0] + '-contour-plot.jpg', 
       format = 'jpg', dpi = dpi, bbox_inches = 'tight'
    )
    plt.show()

    file_name = photo.split(".")[0] + '-results.xlsx'
    df.to_excel(file_name, sheet_name = 'Sheet 1', index = False)

    return df


# testing function
seed_scanner('your/path/to/seeds.jpg', 150, 1000, 300)