density_effects_fx <- function(Db, theta_33, theta_sat, Fo, rho_o, rho_m){
  # to considerer the effect of 'Db' on Saxton and Rawls (2006) soil water coefficients 
  
  # arguments:
  #  - Db: dry soil bulk density (g/cm3) 
  #  - theta_33: soil volumetric water content at 33 kPa (cm3/cm3)
  #  - theta_sat: soil volumetric water content at saturation (cm3/cm3)
  #  - Fo: organic fraction of soil mass
  #  - rho_o: soil organic matter density (g/cm3)
  #  - rho_m: soil mineral matter density (g/cm3)
  
  theta_sat_Db = 1 - ((Db*(1 - Fo))/rho_m) - (Db*Fo/rho_o)
  theta_33_Db = theta_33 - 0.2*(theta_sat - theta_sat_Db)
  theta_sat_33_Db = theta_sat_Db - theta_33_Db
  return(
    data.frame(
      theta_sat_Db = theta_sat_Db, 
      theta_33_Db = theta_33_Db, 
      theta_s33_Db = theta_sat_33_Db
    )
  )
}

water_coefficients_fx <- function(C, S, OM = 0, Db = 0, rho_o = 0, rho_m = 0, correct_by_density = TRUE){
  # Saxton and Rawls (2006) soil water coefficients
  
  # arguments:
  #  - C: percentage of clay in soil mass (kg/kg)
  #  - S: percentage of sand in soil mass (kg/kg)
  #  - OM: percentage of organic matter in soil mass (%)
  #  - Db: dry soil bulk density (g/cm3) 
  #  - Fo: organic fraction of soil mass
  #  - rho_o: soil organic matter density (g/cm3)
  #  - rho_m: soil mineral matter density (g/cm3)
  #  - correct_by_density: use 'density_effects_fx'? (logic)
  
  theta_33t <- -0.251*S + 0.195*C + 0.011*OM + 
    0.006*(S*OM) - 0.027*(C*OM) + 
    0.452*(S*C) + 0.299
  theta_33 <- theta_33t + (1.283*(theta_33t)**2 - 0.374*(theta_33t) - 0.015)
  
  theta_1500t <- -0.024*S + 0.487*C + 0.006*OM + 
    0.005*(S*OM) - 0.013*(C*OM) + 
    0.068*(S*C) + 0.031
  theta_1500 <- abs(theta_1500t + (0.14*theta_1500t - 0.02))
  
  theta_s33t <- 0.278*S + 0.034*C + 0.022*OM -
    0.018*(S*OM) - 0.027*(C*OM) -
    0.584*(S*C) + 0.078
  theta_s33 <- theta_s33t + (0.636*theta_s33t - 0.107)
  
  theta_sat <- theta_33 + theta_s33 - 0.097*S + 0.043 # volumetric water content (theta) at saturation

  # To calculate 'rho_o' if it is not indicated
  Fo <- OM/100
  if(rho_o == 0){
    rho_o <- 1.127 + 0.373*Fo # (Rühlmann et al., 2006) 
  }
  
  # To calculate 'rho_m' if it is not indicated
  if(rho_m == 0){
    rho_m <- 2.65
  }
  
  # To calculate 'Db' if it is not indicated
  if(Db == 0){
    if(Fo == 0){
      Db <- (1 - theta_sat)*rho_m
    }else{
      Fm <- 1 - Fo
      Db <- (1 - theta_sat)*(rho_m*rho_o)/(rho_m*Fo + rho_o*Fm)
    }
  }
  
  if(correct_by_density){
    density_effects <- density_effects_fx(Db = Db, theta_33 = theta_33, theta_sat = theta_sat, Fo = Fo, rho_o = rho_o, rho_m = rho_m)
    theta_sat <- density_effects[['theta_sat_Db']]
    theta_33 <- density_effects[['theta_33_Db']]
    theta_s33 <- density_effects[['theta_s33_Db']]
  }
  
  psiet <- -21.67*S - 27.93*C - 81.97*theta_s33 + 
    71.12*(S*theta_s33) + 8.29*(C*theta_s33) +
    14.05*(S*C) + 27.16
  psie <- psiet + (0.02*psiet**2 - 0.113*psiet - 0.70)
  
  B <- (log(1500) - log(33))/(log(theta_33) - log(theta_1500))
  A <- exp(log(33) + B*log(theta_33))
  
  # saturatedConductivity (mm/h)
  lambda = 1/B
  K_sat = ifelse(theta_sat - theta_33 > 0, yes = 1930*(theta_sat - theta_33)**(3 - lambda), no = 0)
  K_sat = K_sat*2.4 # (cm/h) for simRoot simulations
  
  return(
    data.frame(
      Db = Db,
      Fo = Fo,
      theta_33 = theta_33, 
      theta_1500 = theta_1500,
      theta_s33 = theta_s33,
      psie = psie,
      theta_sat = theta_sat,
      AWE = theta_33 - theta_1500, # Available water holding capacity 
      A = A,
      B = B,
      K_sat = K_sat
    )
  )
}

SWCC_Saxton_and_Rawls_fx <- function(z, EC, theta_33, theta_1500, psie, theta_sat, A, B, resolution = 30){
  # Soil water characteristic curve according to Saxton and Rawls (2006)
  
  # arguments:
  #  - z: rooting depth (m)
  #  - EC: soil electrical conductivity (dS/m)
  #  - theta_33: soil volumetric water content at 33 kPa (cm3/cm3)
  #  - theta_1500: soil volumetric water content at 1500 kPa (cm3/cm3)
  #  - psie: tension at air entry (kPa)
  #  - theta_sat: soil volumetric water content at saturation (cm3/cm3)
  #  - A and B: Saxton and Rawls (2006) soil coefficients (see 'water_coefficients_fx')
  #  - resolution: number of points in the curve
  
  theta_1500_33 <- seq(from = theta_1500, to = theta_33, length = floor(resolution*1/2))
  PSI_1500_33 <- A*(theta_1500_33**-B) 
  
  theta_33_psie <- seq(from = theta_33, to = theta_sat, length = ceiling(resolution*1/2))
  PSI_33_psie <- 33 - ((theta_33_psie - theta_33)*(33 - psie)/(theta_sat - theta_33))
  
  theta <- c(theta_1500_33, theta_33_psie)
  Psi_matric <- c(PSI_1500_33, PSI_33_psie)
  Psi_matric <- ifelse(Psi_matric < 0, yes = 0, no = Psi_matric)
  
  Psi_gravitational <- z*0.01*1000 # kPa
  Psi_osmotic <- ifelse(theta == theta_sat, yes = 36*EC, no = (theta_sat/theta)*36*EC)
  PSI <- Psi_matric + Psi_osmotic + Psi_gravitational
  
  return(
    data.frame(
      theta = theta, 
      PSI = PSI
    )
  )
}

SWCC_van_Genuchten_fx <- function(theta, PSI, theta_1500, theta_sat, alpha = 0.01, n = 2){
  # Soil water characteristic curve according to van Genuchten (1980)
  
  # arguments:
  #  - theta: vector of soil volumetric water content (cm3/cm3)
  #  - PSI: vector of soil water potential (kPa)
  #  - theta_1500: soil volumetric water content at 1500 kPa (cm3/cm3)
  #  - theta_sat: soil volumetric water content at saturation (cm3/cm3)
  #  - alpha and n: coefficients of van Genuchten's (1980) SWCC
  
  starts <- list(
    theta_sat = theta_sat, 
    theta_r = theta_1500, 
    alpha = alpha, 
    n = n
  )
  
  model <- nls(
    theta ~ theta_r + (theta_sat - theta_r)/(1 + (alpha*PSI)^n)^(1 - 1/n),
    algorithm = "port",
    start = starts
  )
  
  coefficients <- data.frame(t(summary(model)[["coefficients"]][, 1]))
  predicted_theta <- predict(model)
  coefficients[["r.squared"]] <- cor(theta, predicted_theta)**2
  
  return(
    list(
      coefficients = coefficients, 
      SWCC = data.frame(theta, PSI, predicted_theta)
    )
  )
}

theta_Psi_at_MAD_fx <- function(z, EC, MAD, theta_33, theta_1500, psie, theta_sat, A, B){
  # theta and PSI at MAD (Management Allowed Depletion)
  
  # arguments:
  #  - z: rooting depth (m)
  #  - EC: soil electrical conductivity (dS/m)
  #  - MAD: Management Allowed Depletion (%), maximum amount of Plant Available Water allowed to be removed from the soil before irrigation refill occurs
  #  - theta_33: soil volumetric water content at 33 kPa (cm3/cm3)
  #  - theta_1500: soil volumetric water content at 1500 kPa (cm3/cm3)
  #  - psie: tension at air entry (kPa)
  #  - theta_sat: soil volumetric water content at saturation (cm3/cm3)
  #  - A and B: Saxton and Rawls (2006) soil coefficients (see 'water_coefficients_fx')
  
  AWE <- theta_33 - theta_1500 # Available water holding capacity 
  theta_mad <- AWE*MAD/100 + theta_1500
  
  if(theta_mad <= theta_33){
    Psi_matric_mad <- A*(theta_mad**-B) 
  }else{
    Psi_matric_mad <- 33 - ((theta_mad - theta_33)*(33 - psie)/(theta_sat - theta_33))
  }
  
  Psi_gravitational_mad <- z*0.01*1000 # kPa
  Psi_osmotic_mad <- (theta_sat/theta_mad)*36*EC
  PSI_mad <- Psi_matric_mad + Psi_osmotic_mad + Psi_gravitational_mad
  
  data.frame(
    list(
      theta = theta_mad, 
      PSI = PSI_mad
    )
  )
}

penetration_resistance_fx <- function(Db, OM, theta_sat, psie, theta_mad, PSI_mad){
  # some equations to calculate the penetration resistance at MAD (Whalley et al. 2007; Gao et al. 2016)
  
  #  - Db: dry soil bulk density (g/cm3) 
  #  - OM: percentage of organic matter in soil mass (%) 
  #  - theta_sat: soil volumetric water content at saturation (cm3/cm3)
  #  - psie: tension at air entry (kPa) 
  #  - theta_mad: soil volumetric water content at MAD (cm3/cm3) 
  #  - PSI: soil water potential at MAD (kPa) 
  
  # Whalley et al. (2005)
  degree_saturation = ifelse(theta_mad/theta_sat > 0.5, yes = theta_mad/theta_sat, no = 0.5) # degree of saturation
  sigma_w = degree_saturation*(PSI_mad) # effective stress
  
  # Equations from Whalley et al. (2007)
  log10Q_eq1 <- 0.27*log10(PSI_mad) + 2.38*Db + 0.14*(OM/1.724) # log10 penetrometer resistance (kPa)
  log10Q_eq2 <- 0.32*log10(sigma_w) + 2.01*Db # log10 penetrometer resistance (kPa)
  log10Q_eq3 <- 0.76*log10(sigma_w) + log10(62) # log10 penetrometer resistance (kPa)
  log10Q_eq4 <- 0.35*log10(sigma_w) + 0.93*Db + 1.2623 # log10 penetrometer resistance (kPa)
  Penetration_mad_eq1 <- 10**log10Q_eq1/1000 # Penetration resistance (MPa)
  Penetration_mad_eq2 <- 10**log10Q_eq2/1000 # Penetration resistance (MPa)
  Penetration_mad_eq3 <- 10**log10Q_eq3/1000 # Penetration resistance (MPa)
  Penetration_mad_eq4 <- 10**log10Q_eq4/1000 # Penetration resistance (MPa)
  
  
  # Equations from Gao et al. (2016)
  F <- 3.73
  A_star <- 1
  f <- 0.1622
  b <- -0.202 
  e <- theta_sat/(1 - theta_sat) # soils void ratio
  Q_eq5 <- Db*9.81*(A_star*(((F - e)^2)/(1 + e))*(PSI_mad*(PSI_mad/psie)^b)^f)^2
  Q_eq6 <- Db*9.81*(A_star*(((F - e)^2)/(1 + e))*((PSI_mad*degree_saturation)^f))^2
  Penetration_mad_eq5 <- Q_eq5/1000 # Penetration resistance (MPa)
  Penetration_mad_eq6 <- Q_eq6/1000 # Penetration resistance (MPa)
  
  results <- data.frame(
    Penetration_mad_eq1,
    Penetration_mad_eq2,
    Penetration_mad_eq3,
    Penetration_mad_eq4,
    Penetration_mad_eq5,
    Penetration_mad_eq6
  )
  return(results)
}

